------------------------------------------------
--
-- tcl-tk.adb -- 
--
-- Copyright (c) 1995-1996 Terry J. Westley
--
-- See the file "license_terms.html" for information on usage and
-- redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
--
------------------------------------------------

with Tcl.Ada; use Tcl.Ada;
with Text_IO;

package body Tcl.Tk is

   Trace : Boolean := False;

   procedure Tcl_Eval (
      interp          : in Tcl_Interp;
      cmd             : in String) is
   begin -- Tcl_Eval
      if Trace then
         Text_IO.Put_Line (Cmd);
      end if;
      Tcl.Ada.Tcl_Eval (Interp, Cmd);
   end Tcl_Eval;

   procedure Set_Trace (State : in Boolean) is
   begin -- Set_Trace
      Trace := State;
   end Set_Trace;

   function Widget_Image (Win : in Widget'Class) return String is
   begin -- Widget_Image
      return CHelper.Value (Win.Name);
   end Widget_Image;

   function "&" (Left : in Widget'Class; Right : in Widget'Class) return String is
   begin -- "&"
      return Widget_Image (Left) & Widget_Image (Right);
   end "&";

   function "&" (Left : in Widget'Class; Right : in String) return String is
   begin -- "&"
      return Widget_Image (Left) & Right;
   end "&";

   function "&" (Left : in String; Right : in Widget'Class) return String is
   begin -- "&"
      return Left & Widget_Image (Right);
   end "&";

   procedure Set_Context (Interp : in Tcl_Interp) is
   begin -- Set_Context
      Context := Interp;
   end Set_Context;

   function Get_Context return Tcl_Interp is
   begin -- Get_Context
      return Context;
   end Get_Context;

   function Get_Interp (Widgt : in Widget'Class) return Tcl_Interp is
   begin -- Get_Interp
      return Widgt.Interp;
   end Get_Interp;

   procedure Destroy (Widgt : in out Widget'Class) is
   begin -- Destroy
      Execute_Widget_Command (Widgt, "destroy");
      C.Strings.Free (Widgt.Name);
   end Destroy;

   function cget (
      Widgt : in Widget'Class;
      option   : in String) return String is
   begin -- cget
      Execute_Widget_Command (Widgt, "cget", option);
      return Tcl_GetResult (Widgt.Interp);
   end cget;

   function configure (
      Widgt : in Widget'Class;
      options  : in String := "") return String is
   begin -- configure
      Execute_Widget_Command (Widgt, "configure", options);
      return Tcl_GetResult (Widgt.Interp);
   end configure;

   procedure configure (
      Widgt : in Widget'Class;
      options  : in String := "") is
   begin -- configure
      Execute_Widget_Command (Widgt, "configure", options);
   end configure;

   procedure Bind (
      Widgt    : in Widget'class;
      Sequence : in String;
      Script   : in String) is
   begin -- Bind
      Tcl_Eval (Widgt.Interp,
      	 "bind " & Widget_Image (Widgt) & " " & Sequence & " " & Script);
   end Bind;

   procedure Bind (
      Widgt    : in Widget'class;
      Sequence : in String) is
   begin -- Bind
      Tcl_Eval (Widgt.Interp,
      	 "bind " & Widget_Image (Widgt) & " " & Sequence);
   end Bind;

   function Bind (
      Widgt    : in Widget'class;
      Sequence : in String) return String is
   begin -- Bind
      Tcl_Eval (Widgt.Interp,
      	 "bind " & Widget_Image (Widgt) & " " & Sequence);
      return Tcl_GetResult (Widgt.Interp);
   end Bind;

   procedure Bind_to_Main_Window (
      Interp   : in Tcl_Interp;
      Sequence : in String;
      Script   : in String) is
   begin -- Bind_to_Main_Window
      Tcl_Eval (Interp,
      	 "bind . " & Sequence & " " & Script);
   end Bind_to_Main_Window;

   procedure Bind_to_Main_Window (
      Interp   : in Tcl_Interp;
      Sequence : in String) is
   begin -- Bind_to_Main_Window
      Tcl_Eval (Interp,
      	 "bind . " & Sequence);
   end Bind_to_Main_Window;

   function Bind_to_Main_Window (
      Interp   : in Tcl_Interp;
      Sequence : in String) return String is
   begin -- Bind_to_Main_Window
      Tcl_Eval (Interp,
      	 "bind . " & Sequence);
      return Tcl_GetResult (Interp);
   end Bind_to_Main_Window;

   function Create (
      pathName : in String;
      options  : in String := "") return Frame is
   begin -- Create
      return Create (Context, pathName, options);
   end Create;

   function Create (
      Interp   : in Tcl_Interp;
      pathName : in String;
      options  : in String := "") return Frame is
   --
      The_Widget : Frame;
   begin -- Create
      The_Widget.Interp := Interp;
      The_Widget.Name   := C.Strings.New_String (pathName);
      Tcl_Eval (The_Widget.Interp, "frame " & pathName & " " & options);
      return The_Widget;
   end Create;

   function Get_Main_Window (
      Interp : in Tcl_Interp) return Frame is
   begin -- Get_Main_Window
      return (C.Strings.New_String ("."), Interp);
   end Get_Main_Window;

   function Create (
      pathName : in String;
      options  : in String := "") return Toplevel is
   begin -- Create
      return Create (Context, pathName, options);
   end Create;

   function Create (
      Interp   : in Tcl_Interp;
      pathName : in String;
      options  : in String := "") return Toplevel is
   --
      The_Widget : Toplevel;
   begin -- Create
      The_Widget.Interp := Interp;
      The_Widget.Name   := C.Strings.New_String (pathName);
      Tcl_Eval (The_Widget.Interp, "toplevel " & pathName & " " & options);
      return The_Widget;
   end Create;

   function Create (
      pathName : in String;
      options  : in String := "") return Label is
   begin -- Create
      return Create (Context, pathName, options);
   end Create;

   function Create (
      Interp   : in Tcl_Interp;
      pathName : in String;
      options  : in String := "") return Label is
   --
      The_Widget : Label;
   begin -- Create
      The_Widget.Interp := Interp;
      The_Widget.Name   := C.Strings.New_String (pathName);
      Tcl_Eval (The_Widget.Interp, "label " & pathName & " " & options);
      return The_Widget;
   end Create;

   function Create (
      pathName : in String;
      options  : in String := "") return Message is
   begin -- Create
      return Create (Context, pathName, options);
   end Create;

   function Create (
      Interp   : in Tcl_Interp;
      pathName : in String;
      options  : in String := "") return Message is
   --
      The_Widget : Message;
   begin -- Create
      The_Widget.Interp := Interp;
      The_Widget.Name   := C.Strings.New_String (pathName);
      Tcl_Eval (The_Widget.Interp, "message " & pathName & " " & options);
      return The_Widget;
   end Create;

   function Create (
      pathName : in String;
      options  : in String := "") return Button is
   begin -- Create
      return Create (Context, pathName, options);
   end Create;

   function Create (
      Interp   : in Tcl_Interp;
      pathName : in String;
      options  : in String := "") return Button is
   --
      The_Widget : Button;
   begin -- Create
      The_Widget.Interp := Interp;
      The_Widget.Name   := C.Strings.New_String (pathName);
      Tcl_Eval (The_Widget.Interp, "button " & pathName & " " & options);
      return The_Widget;
   end Create;

   function Create (
      pathName : in String;
      options  : in String := "") return Radiobutton is
   begin -- Create
      return Create (Context, pathName, options);
   end Create;

   function Create (
      Interp   : in Tcl_Interp;
      pathName : in String;
      options  : in String := "") return Radiobutton is
   --
      The_Widget : Radiobutton;
   begin -- Create
      The_Widget.Interp := Interp;
      The_Widget.Name   := C.Strings.New_String (pathName);
      Tcl_Eval (The_Widget.Interp, "radiobutton " & pathName & " " & options);
      return The_Widget;
   end Create;

   procedure Flash (
      Buttn : in Button'class) is
   begin -- Flash
      Execute_Widget_Command (Widget (Buttn), "flash");
   end Flash;

   function Invoke (
      Buttn : in Button'class;
      options  : in String := "") return String is
   begin -- Invoke
      Execute_Widget_Command (Widget (Buttn), "invoke", options);
      return Tcl_GetResult (Buttn.Interp);
   end Invoke;

   procedure Deselect (
      Buttn : in RadioButton) is
   begin -- Deselect
      Execute_Widget_Command (Widget (Buttn), "deselect");
   end Deselect;

   procedure Tk_Select (
      Buttn : in RadioButton) is
   begin -- Tk_Select
      Execute_Widget_Command (Widget (Buttn), "select");
   end Tk_Select;

   procedure Toggle (
      Buttn : in RadioButton) is
   begin -- Toggle
      Execute_Widget_Command (Widget (Buttn), "toggle");
   end Toggle;

   procedure After (
      Ms     : in Natural) is
   begin -- After
      Tcl_Eval (Tk.Context, "after " & Natural'image (Ms));
   end After;

   procedure After (
      Interp : in Tcl_Interp;
      Ms     : in Natural) is
   begin -- After
      Tcl_Eval (Interp, "after " & Natural'image (Ms));
   end After;

   function  After (
      Ms     : in Natural;
      Script : in String) return String is
   begin -- After
      Tcl_Eval (Tk.Context, "after " & Natural'image (Ms) & " " & Script);
      return Tcl_GetResult (Tk.Context);
   end After;

   procedure After (
      Ms     : in Natural;
      Script : in String) is
   begin -- After
      Tcl_Eval (Tk.Context, "after " & Natural'image (Ms) & " " & Script);
   end After;

   function  After (
      Interp : in Tcl_Interp;
      Ms     : in Natural;
      Script : in String) return String is
   begin -- After
      Tcl_Eval (Interp, "after " & Natural'image (Ms) & " " & Script);
      return Tcl_GetResult (Tk.Context);
   end After;

   procedure After (
      Interp : in Tcl_Interp;
      Ms     : in Natural;
      Script : in String) is
   begin -- After
      Tcl_Eval (Interp, "after " & Natural'image (Ms) & " " & Script);
   end After;

   procedure Cancel (
      id_or_script : in String) is
   begin -- Cancel
      Tcl_Eval (Tk.Context, "after cancel " & id_or_script);
   end Cancel;

   procedure Cancel (
      Interp       : in Tcl_Interp;
      id_or_script : in String) is
   begin -- Cancel
      Tcl_Eval (Interp, "after cancel " & id_or_script);
   end Cancel;

   function  Idle (
      Script : in String) return String is
   begin -- Idle
      Tcl_Eval (Tk.Context, "after idle " & Script);
      return Tcl_GetResult (Tk.Context);
   end Idle;

   procedure Idle (
      Script : in String) is
   begin -- Idle
      Tcl_Eval (Tk.Context, "after idle " & Script);
   end Idle;

   function  Idle (
      Interp : in Tcl_Interp;
      Script : in String) return String is
   begin -- Idle
      Tcl_Eval (Interp, "after idle " & Script);
      return Tcl_GetResult (Tk.Context);
   end Idle;

   procedure Idle (
      Interp : in Tcl_Interp;
      Script : in String) is
   begin -- Idle
      Tcl_Eval (Interp, "after idle " & Script);
   end Idle;

   function Info (
      id     : in String := "") return String is
   begin -- Info
      Tcl_Eval (Tk.Context, "after info " & id);
      return Tcl_GetResult (Tk.Context);
   end Info;

   function Info (
      Interp : in Tcl_Interp;
      id     : in String := "") return String is
   begin -- Info
      Tcl_Eval (Interp, "after info " & id);
      return Tcl_GetResult (Tk.Context);
   end Info;

   procedure Pack (
      Slave   : in Widget'Class;
      Options : in String) is
   begin -- Pack
      Tcl_Eval (Slave.Interp, "pack " & Widget_Image (Slave) & " " & Options);
   end Pack;

   procedure Pack_Configure (
      Slave   : in Widget'Class;
      Options : in String) is
   begin -- Pack_Configure
      Tcl_Eval (Slave.Interp, "pack configure " & Widget_Image (Slave) &
      	 " " & Options);
   end Pack_Configure;

   procedure Pack_Forget (
      Slave   : in Widget'Class) is
   begin -- Pack_Forget
      Tcl_Eval (Slave.Interp, "pack forget " & Widget_Image (Slave));
   end Pack_Forget;

   function Pack_Info (
      Slave   : in Widget'Class) return String is
   begin -- Pack_Info
      Tcl_Eval (Slave.Interp, "pack info " & Widget_Image (Slave));
      return Tcl_GetResult (Slave.Interp);
   end Pack_Info;

   procedure Pack_Propogate (
      Master  : in Widget'Class;
      State   : in Boolean) is
   begin -- Pack_Propogate
      Tcl_Eval (Master.Interp, "pack propogate " & Widget_Image (Master) &
         " " & Integer'image (Boolean'pos (State)));
   end Pack_Propogate;

   function Pack_Propogate (
      Master  : in Widget'Class) return Boolean is
   begin -- Pack_Propogate
      Tcl_Eval (Master.Interp, "pack propogate " & Widget_Image (Master));
      return Integer'value (Tcl_GetResult (Master.Interp)) = 1;
   end Pack_Propogate;

   function Pack_Slaves (
      Master  : in Widget'Class) return String is
   begin -- Pack_Slaves
      Tcl_Eval (Master.Interp, "pack slaves " & Widget_Image (Master));
      return Tcl_GetResult (Master.Interp);
   end Pack_Slaves;

   procedure Execute_Widget_Command (
      Widgt : in Widget'Class;
      command  : in String;
      options  : in String := "") is
   begin -- Execute_Widget_Command
      Tcl_Eval (Widgt.Interp,
      	 Widget_Image (Widgt) & " " & command & " " & options);
   end Execute_Widget_Command;

end Tcl.Tk;
